             .title        "Sunsoft-2 (INES 89 / INES 93) Plugin"

;11/22/07
;BootGod
;Note: Because only 1 game is known to use this board, plugin is currently
;setup specifically for it, i.e. fixed to 128K PRG and 128K CHR w/ mapper
;controlled mirroring. Board supports hardwired mirroring.

;02/17/09
;BootGod
;Implemented support for m93 which is fundamentaly the same as m89
;Added mirroring check to detect between 1-screen (mpc) and H / V (hw)
;Added generic routine for selecting bank with bus conflicts

             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
curbank:    .equ 00e2h
banksleft:    .equ 00e3h
bank0crc:  .equ 00e4h

reg:       .equ 00e8h
reg_lo:    .equ 00e8h
reg_hi:    .equ 00e9h

numpages_lo .equ 00eah
numpages_hi .equ 00ebh

x_temp:      .equ 00ech
y_temp:      .equ 00edh
has_vram:    .equ 00eeh


;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Sunsoft-2 Carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 128 >> 4,0 >> 4
             .db 128 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             lda #1		;default to has VRAM
             sta has_vram

;check mirroring
             ldx #02fh
init_md:     txa		;load first byte of 2000, 2400, 2800, 2c00 w/ 00h
             jsr wr_ppu
             lda #0
             sta 2007h
             dex
             cpx #01fh
             bne init_md

             ldx #0		;start off with check for H mirroring

             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h		;write a value to $2000

             lda #028h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             beq chk_1scr	;if value appears at $2800, then we have H mirroring

             inx		;next check for V mirroring

             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             beq chk_1scr	;if value appears at $2400, then we have V mirroring

             inx		;it wasn't mirrorred so it's 4 screen
             bne send_mirr
             
chk_1scr:    lda #02ch		;check if it was mirrored to all NT's
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne send_mirr

             ldx #04h		;it was, so is 1-screen mapper controlled
             lda #0		;flag that it has CHR-ROM
             sta has_vram

send_mirr:   txa
             jsr send_byte


;send PRG size info

got_prg:     lda #0		;send PRG header
             jsr send_byte
             lda #2		;200h pages = 128K
             jsr send_byte
             lda #001h
             jsr send_byte

             lda #0
             sta curbank
             lda #8
             sta banksleft	;8 x 16K

             ldy #0

next_prg:    lda curbank	
             asl a
             asl a
             asl a
             asl a
             jsr wr_bank

             lda #0
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             ldy #0
             ldx #64		;setup for 16K bank dump

dump_prg:    lda (address),y
             jsr send_byte
             iny
             bne dump_prg
             inc address_hi
             dex
             bne dump_prg
             inc curbank
             dec banksleft
             bne next_prg
             

;send CHR ROM size info
             ;jsr chk_vram	;this doesn't work for some reason
             ;beq done
             lda #1		;so use the mirroring flag instead
             cmp has_vram
             beq done
             
got_chr:     
             lda #0
             sta curbank
             lda #16
             sta banksleft	;16 x 8K

             lda #0		;send CHR header 
             jsr send_byte
             lda #2		;200h pages = 128K
             jsr send_byte
             lda #002h
             jsr send_byte
             
next_chr:    lda curbank
             cmp #8
             bcc lower
             ora #080h
lower:       jsr wr_bank	;select bank at PPU $0000

             lda #0
             jsr wr_ppu
             lda 2007h

             ldx #32		;32 pages = 8K
             ldy #0

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             inc curbank
             dec banksleft
             bne next_chr

done:        lda #000h		;send termination header
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte

             rts

;sub for setting CHR bank (compensates for bus conflicts)
wr_bank:     stx x_temp
             sty y_temp

             sta reg		;store value to be written to reg

             lda #0
             sta address_lo
             lda #0C0h
             sta address_hi	;set address to $C000

             ldx #040h		;setup to search last bank (16K)
             ldy #0

wrb_loop:    lda (address),y
             and #0f7h		;mask it to just the bits we need to match
             cmp reg		;check if current byte matches what we want to write
             beq found_reg
             iny
             bne wrb_loop
             inc address_hi
             bne wrb_loop

no_reg:      lda reg		;didn't find place to write, try it anyways
             sta 0c000h
             jmp wrb_return

found_reg:   lda (address),y	;found value equal to reg, write it.
             sta (address),y

wrb_return:  ldx x_temp
             ldy y_temp

             rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
